unit MainForm;

interface

uses NTypes, NForm, NCommand, NConf, NCanvas;

var
  Command: TCommand;
  Config: TConf;
  ProfDir: string;

type
  TDate = record
    Year, Month, Day, YearDay: Integer;
    Holiday: Boolean;
  end;  

  TCalendar = array[0..41] of TDate; 

  TNkCalMainForm = class(TForm)
    BgColor: Integer;
    Calendar: TCalendar;
    DayColor: array[0..6] of Integer;
    FirstWDay: Integer;
    FontBold: Boolean;
    FontItalic: Boolean;
    FontName: string;
    HolidayColor: Integer;
    Month: Integer;
    OldDay: Integer;
    OuterDayColor: Integer;
    OuterDays: Boolean;
    Timer: TTimer;
    Today: TTime;
    TodayColor: Integer;
    WeekNumber: Boolean;
    WeekNumberColor: Integer;
    Year: Integer;
    YearDay: Boolean;
    constructor Create; override;
    destructor Destroy; override;
    procedure Closing; override;
    procedure DispatchCommand(Id: Longint); override;
    procedure Paint(const Canvas: TCanvas); override;
    procedure ShowContextMenu(SX, SY: Longint); override;
    procedure UpdateCalendar;
    procedure UpdateToday;
  published
    procedure cmAbout;
    procedure cmExit;
    procedure cmNextMonth;
    procedure cmNextYear;
    procedure cmPrevMonth;
    procedure cmPrevYear;
    procedure cmShowWeekNumber;
    procedure cmShowYearDay;
    procedure cmStayOnTop;
    procedure cmThisMonth;
    procedure cmZoom100;
    procedure cmZoom150;
    procedure cmZoom200;
    procedure cmZoom300;
  end;

implementation

uses NLib, NDate, Version, Menu, holiday_jp;

const
  DayName: array[0..6] of string = (
    'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'
  );

function GetCalendar(Year, Month, FirstWeekDay: Integer): TCalendar;
var
  d, i: Integer;
  wd1, wde, py, pm, pdm, prev, ny, nm: Integer;
begin
  FillChar(Result, SizeOf(Result), 0);

  if Month = 0 then
    Exit;

  wd1 := WeekDay(Year, Month, 1);
  wde := WeekDay(Year, Month, DaysInMonth(Year, Month));
  i := 0;
  
  // previous month days
  prev := (wd1 - FirstWeekDay + 7) mod 7;
  if Month = 1 then
  begin
    py := Year-1;
    pm := 12;
  end
  else begin
    py := Year;
    pm := Month-1;
  end;
  pdm := DaysInMonth(py, pm);
  for d := pdm-prev+1 to pdm do
  begin
    Result[i].Year := py;
    Result[i].Month := pm;
    Result[i].Day := d;
    Result[i].YearDay := YearDay(py, pm ,d);
    Inc(i);
  end;

  // this month days
  for d := 1 to DaysInMonth(Year, Month) do
  begin
    Result[i].Year := Year;
    Result[i].Month := Month;
    Result[i].Day := d;
    Result[i].YearDay := YearDay(Year, Month, d);
    Inc(i);
  end;

  // next month days
  if Month = 12 then
  begin
    ny := Year+1;
    nm := 1;
  end
  else begin
    ny := Year;
    nm := Month+1;
  end;
  for d := 1 to (FirstWeekday - wde + 6) mod 7 do
  begin
    Result[i].Year := ny;
    Result[i].Month := nm;
    Result[i].Day := d;
    Result[i].YearDay := YearDay(ny, nm ,d);
    Inc(i);
  end;
end;

constructor TNkCalMainForm.Create;
var
  arg1, arg2: string;
begin
  inherited;

  BgColor := $ffffff;
  TodayColor := $e6eaf2;
  HolidayColor := $ff0000;
  DayColor[0] := 0;
  DayColor[1] := 0;
  DayColor[2] := 0;
  DayColor[3] := 0;
  DayColor[4] := 0;
  DayColor[5] := $4848f0;
  DayColor[6] := $f04848;
  OuterDayColor := $e0e0e0;
  WeekNumberColor := $a0a0a0;
  
  SetBackground(BgColor);
  
  LoadFormPos(Self, Config);
  FirstWDay := Config.GetInt('firstweekday');
  FontName := Config.GetStr('font');
  arg1 := Config.GetStr('font', 1);
  arg2 := Config.GetStr('font', 2);
  FontBold := (arg1 = 'bold') or (arg2 = 'bold');
  FontItalic := (arg1 = 'italic') or (arg2 = 'italic');
  OuterDays := Config.GetBool('outerdays');
  WeekNumber := Config.GetBool('weeknumber');
  YearDay := Config.GetBool('yearday');

  UpdateToday;
  cmThisMonth;
  Timer := TTimer.Create(Self, UpdateToday, 1000);
  Timer.Start;

  Show;
end;

destructor TNkCalMainForm.Destroy;
begin
  Timer.Free;
  inherited;
end;

procedure TNkCalMainForm.ShowContextMenu(SX, SY: Longint);
begin
  with TPopupMenu.Create do
    try
      Command.AddMenuItems(Handle, MenuItems);
      CheckItem(Command.GetId(cmStayOnTop), StayOnTop);
      CheckItem(Command.GetId(cmShowWeekNumber), WeekNumber);
      CheckItem(Command.GetId(cmShowYearDay), YearDay);
      Popup(Self.Handle, SX, SY);
    finally
      Free;
    end;
end;

procedure TNkCalMainForm.DispatchCommand(Id: Longint);
begin
  Command.DispatchCommand(Id);
end;

procedure TNkCalMainForm.Paint(const Canvas: TCanvas);
var
  d: TDate;
  cw, ch, h, dw, x, y, i, j, wd: Integer;
  fontsize, daynamefontsize, yeardayfontsize: Integer;
begin
  GetClientSize(cw, ch);
  h := ch div 8;
  fontsize := h * 90 div 100;
  daynamefontsize := fontsize * 3 div 4;
  yeardayfontsize := fontsize div 3;

  Canvas.BeginDraw;

  // year and month
  if (Year = Today.Year) and (Month = Today.Mon) then
    Canvas.FillRect(0, 0, cw, h, TodayColor);
  Canvas.SetFont(FontName, fontsize, FontBold, FontItalic);
  Canvas.SetTextAlign(taCenter);
  Canvas.DrawText(cw div 2, (h-fontsize) div 2,
      Format('{0}/{1}', [Year, Month]));

  // day name
  y := h + (h-daynamefontsize) div 2;
  Canvas.SetFont(FontName, daynamefontsize, FontBold, FontItalic);
  if WeekNumber then
  begin
    dw := cw div 8;
    x := dw div 2;
    Canvas.SetTextColor(WeekNumberColor);
    Canvas.DrawText(x, y, 'Wk');
    Inc(x, dw);
  end
  else begin
    dw := cw div 7;
    x := dw div 2;
  end;
  for i := 0 to 6 do
  begin
    j := (FirstWDay+i) mod 7;
    Canvas.SetTextColor(DayColor[j]);
    Canvas.DrawText(x, y, DayName[j]);
    Inc(x, dw);
  end;

  // days
  Canvas.SetFont(FontName, fontsize, FontBold, FontItalic);
  wd := FirstWDay;
  y := h*2;
  for i := 0 to 5 do
  begin
    if WeekNumber then
      x := dw
    else
      x := 0;
    for j := 0 to 6 do
    begin
      d := Calendar[i*7+j];
      if d.Day = 0 then
        Break;
      
      if WeekNumber and (wd = 0) then
      begin
        Canvas.SetFont(FontName, daynamefontsize, FontBold, FontItalic);
        Canvas.SetTextColor(WeekNumberColor);
        Canvas.DrawText(dw div 2, y + (h-daynamefontsize) div 2,
            IStr(ISOWeekNumber(d.Year, d.Month, d.Day)));
        Canvas.SetFont(FontName, fontsize, FontBold, FontItalic);
      end;

      if d.Month = Month then
      begin
        if IsHoliday(d.Year, d.Month, d.Day) then
          Canvas.SetTextColor(HolidayColor)
        else
          Canvas.SetTextColor(DayColor[wd]);
      end
      else if OuterDays then
        Canvas.SetTextColor(OuterDayColor)
      else
        Canvas.SetTextColor(BgColor);

      if ((d.Year=Year) and (d.Month=Month) and (d.Day=Today.MDay) and 
           (Year=Today.Year) and (Month=Today.Mon)) or
         ((d.Year=Today.Year) and (d.Month=Today.Mon) and
           (d.Day=Today.MDay) and OuterDays) then
        Canvas.FillRect(x, y, dw, h, TodayColor);

      if YearDay then 
      begin
        Canvas.DrawText(x + dw div 2, y, IStr(d.Day));
        if d.Month = Month then
        begin
          Canvas.SetFont(FontName, yeardayfontsize, FontBold, FontItalic);
          Canvas.SetTextColor(WeekNumberColor);
          Canvas.DrawText(x + dw div 2, y + h - yeardayfontsize,
            Format('{0}-{1}', [d.YearDay, 365+Ord(IsLeap(d.Year))-d.YearDay]));
          Canvas.SetFont(FontName, fontsize, FontBold, FontItalic);
        end;
      end else
        Canvas.DrawText(x + dw div 2, y + (h-fontsize) div 2, IStr(d.Day));
      wd := (wd+1) mod 7;
      Inc(x, dw);
    end;
    Inc(y, h);
  end;
  
  Canvas.EndDraw;
end;

procedure TNkCalMainForm.Closing;
var
  position: TConf;
begin
  position := TConf.Create;
  try
    SaveFormPos(Self, position);
    try
      position.SaveToFile(JoinPath([ProfDir, 'position']));
    except
      on E: EIOError do App.Error(E.Path+#10#10+E.Msg);
    end;
  finally
    position.Free;
  end;
end;

procedure TNkCalMainForm.UpdateToday;
begin
  OldDay := Today.MDay;
  Today := LocalTime(Time);
  if OldDay <> Today.MDay then 
    UpdateCalendar;
end;

procedure TNkCalMainForm.UpdateCalendar;
begin
  Calendar := GetCalendar(Year, Month, FirstWDay);
  SetText(Format('{0}/{1} - {2}', [Year, Month, App.Name]));
  Invalidate;
end;

{ Command Methods }

procedure TNkCalMainForm.cmStayOnTop;
begin
  SetStayOnTop(not StayOnTop);
end;

procedure TNkCalMainForm.cmPrevMonth;
begin
  if Month = 1 then
  begin
    Month := 12;
    Dec(Year);
  end else
    Dec(Month);
  UpdateCalendar;
end;

procedure TNkCalMainForm.cmNextMonth;
begin
  if Month = 12 then
  begin
     Month := 1;
     Inc(Year);
  end else
     Inc(Month);
  UpdateCalendar;
end;

procedure TNkCalMainForm.cmThisMonth;
begin
  Year := Today.Year;
  Month := Today.Mon; 
  UpdateCalendar;
end;

procedure TNkCalMainForm.cmNextYear;
begin
  Inc(Year);
  UpdateCalendar;
end;

procedure TNkCalMainForm.cmPrevYear;
begin
  Dec(Year);
  UpdateCalendar;
end;

procedure TNkCalMainForm.cmZoom100;
begin
  SetClientSize(320, 240);
end;

procedure TNkCalMainForm.cmZoom150;
begin
  SetClientSize(480, 360);
end;

procedure TNkCalMainForm.cmZoom200;
begin
  SetClientSize(640, 480);
end;

procedure TNkCalMainForm.cmZoom300;
begin
  SetClientSize(960, 720);
end;

procedure TNkCalMainForm.cmShowWeekNumber;
begin
  WeekNumber := not WeekNumber;
  Invalidate;
end;

procedure TNkCalMainForm.cmShowYearDay;
begin
  YearDay := not YearDay;
  Invalidate;
end;

procedure TNkCalMainForm.cmAbout;
begin
  App.About(
    App.Name+' '+Version_+' ('+ReleaseDate+')'#10+
    #10+
    Copyright+#10+
    'This software is provided under the zlib/libpng license.'#10+
    #10+
    'Profile Directory: '+ProfDir);
end;

procedure TNkCalMainForm.cmExit;
begin
  Close;
end;

end.
